package article_test

import (
	"fmt"
	"net/http"
	"net/http/httptest"
	"testing"
)

func noopTest(t *testing.T) {}

var everythingIsOkayHandler = http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
	w.WriteHeader(http.StatusOK)
})

func TestSequentialSubtests(t *testing.T) {
	// 通常の、直列順サブテストが実行される例
	t.Run("Test1", noopTest)
	t.Run("Test2", noopTest)
	t.Run("Test3", noopTest)
	t.Run("Test4", noopTest)
}

func TestParallelHTTPRequest(t *testing.T) {
	// 全てのサブテストが並行に走る例

	srv := httptest.NewServer(everythingIsOkayHandler)
	defer srv.Close()

	client := srv.Client()
	t.Run("Request1", func(t *testing.T) {
		t.Parallel()
		_, err := client.Get("http://gihyo.jp/foo")
		if err != nil {
			t.Errorf("error: %s", err)
		}
	})
	t.Run("Request2", func(t *testing.T) {
		t.Parallel()
		_, err := client.Get("http://gihyo.jp/bar")
		if err != nil {
			t.Errorf("error: %s", err)
		}
	})
	t.Run("Request3", func(t *testing.T) {
		t.Parallel()
		_, err := client.Get("http://gihyo.jp/baz")
		if err != nil {
			t.Errorf("error: %s", err)
		}
	})
}

func TestMixedParalleHTTPRequestl(t *testing.T) {
	// セットアップ→並列テスト→テアダウンを順番に行う例
	// 途中サブテスト外のコードもあるので、実行順が確認できます。

	// 本来deferで事足りますが、ここでは例として、httptest.Serverを
	// サブテストの中で作成します。今回はエラーが起こりえないので
	// セットアップ・テアダウンでは特にテストはしていませんが、
	// 通常はここでエラーを確認すべきでしょう
	var srv *httptest.Server
	var client *http.Client
	t.Run("Setup Server", func(t *testing.T) {
		srv = httptest.NewServer(everythingIsOkayHandler)
		client = srv.Client()
		t.Logf("server created")
	})

	t.Logf("Preparing parallel tests...")

	t.Run("Run parallel requests", func(t *testing.T) {
		for i := 0; i < 10; i++ {
			t.Logf("Preparing parallel test %d", i)
			t.Run(fmt.Sprintf("Request %d", i), func(t *testing.T) {
				t.Parallel()

				res, err := client.Get(`http://gihyo.jp/`)
				if err != nil {
					t.Errorf("errror: %s", err)
				}

				if res.StatusCode != http.StatusOK {
					t.Errorf("invalid response status code: %d", res.StatusCode)
				}
			})
		}
	})

	t.Logf("All parallel tests done")

	t.Run("Teardown Server", func(t *testing.T) {
		srv.Close()
		t.Logf("server closed")
	})
}
